var EventEmitter, OutgoingMessage, Query, binaryConverters, fieldConverters, stringConverters;
var __hasProp = Object.prototype.hasOwnProperty, __extends = function(child, parent) {
  for (var key in parent) { if (__hasProp.call(parent, key)) child[key] = parent[key]; }
  function ctor() { this.constructor = child; }
  ctor.prototype = parent.prototype;
  child.prototype = new ctor;
  child.__super__ = parent.prototype;
  return child;
};
EventEmitter = require('events').EventEmitter;
OutgoingMessage = require('./frontend_message');
Query = (function() {
  __extends(Query, EventEmitter);
  function Query(connection, sql, callback) {
    this.connection = connection;
    this.sql = sql;
    this.callback = callback;
  }
  Query.prototype.execute = function() {
    this.emit('start');
    if (this.callback) {
      this.rows = [];
    }
    this.connection._writeMessage(new OutgoingMessage.Query(this.sql));
    this.connection.once('EmptyQueryResponse', this.onEmptyQueryListener = this.onEmptyQuery.bind(this));
    this.connection.on('RowDescription', this.onRowDescriptionListener = this.onRowDescription.bind(this));
    this.connection.on('DataRow', this.onDataRowListener = this.onDataRow.bind(this));
    this.connection.on('CommandComplete', this.onCommandCompleteListener = this.onCommandComplete.bind(this));
    this.connection.once('ErrorResponse', this.onErrorResponseListener = this.onErrorResponse.bind(this));
    return this.connection.once('ReadyForQuery', this.onReadyForQueryListener = this.onReadyForQuery.bind(this));
  };
  Query.prototype.onEmptyQuery = function() {
    if (!this.callback) {
      this.emit('error', "The query was empty!");
    }
    if (this.callback) {
      return this.callback("The query was empty!");
    }
  };
  Query.prototype.onRowDescription = function(msg) {
    var column, field, _i, _len, _ref;
    this.fields = [];
    _ref = msg.columns;
    for (_i = 0, _len = _ref.length; _i < _len; _i++) {
      column = _ref[_i];
      field = new Query.Field(column);
      this.emit('field', field);
      this.fields.push(field);
    }
    return this.emit('fields', this.fields);
  };
  Query.prototype.onDataRow = function(msg) {
    var index, row, value, _len, _ref;
    row = [];
    _ref = msg.values;
    for (index = 0, _len = _ref.length; index < _len; index++) {
      value = _ref[index];
      row.push(value != null ? this.fields[index].convert(value) : null);
    }
    if (this.callback) {
      this.rows.push(row);
    }
    return this.emit('row', row);
  };
  Query.prototype.onReadyForQuery = function(msg) {
    return this._removeAllListeners();
  };
  Query.prototype.onCommandComplete = function(msg) {
    this.emit('end', msg.status);
    if (this.callback) {
      this.callback(null, this.fields, this.rows, msg.status);
    }
    if (this.callback) {
      return this.rows = [];
    }
  };
  Query.prototype.onErrorResponse = function(msg) {
    this._removeAllListeners();
    if (!this.callback) {
      this.emit('error', msg);
    }
    if (this.callback) {
      return this.callback(msg.message);
    }
  };
  Query.prototype._removeAllListeners = function() {
    this.connection.removeListener('EmptyQueryResponse', this.onEmptyQueryListener);
    this.connection.removeListener('RowDescription', this.onRowDescriptionListener);
    this.connection.removeListener('DataRow', this.onDataRowListener);
    this.connection.removeListener('CommandComplete', this.onCommandCompleteListener);
    this.connection.removeListener('ErrorResponse', this.onErrorResponseListener);
    return this.connection.removeListener('ReadyForQuery', this.onReadyForQueryListener);
  };
  return Query;
})();
stringConverters = {
  string: function(value) {
    return value.toString();
  },
  integer: function(value) {
    return parseInt(value);
  },
  float: function(value) {
    return parseValue(value);
  },
  bool: function(value) {
    return value.toString() === 't';
  },
  datetime: function(value) {
    var day, hour, minute, month, second, year;
    year = parseInt(value.slice(0, 4));
    month = parseInt(value.slice(5, 7));
    day = parseInt(value.slice(8, 10));
    hour = parseInt(value.slice(11, 13));
    minute = parseInt(value.slice(14, 16));
    second = parseInt(value.slice(17, 19));
    return new Date(Date.UTC(year, month, day, hour, minute, second));
  },
  date: function(value) {
    var day, month, year;
    year = parseInt(value.slice(0, 4));
    month = parseInt(value.slice(5, 7));
    day = parseInt(value.slice(8, 10));
    return new Date(Date.UTC(year, month, day));
  },
  "default": function(value) {
    return value.toString();
  }
};
binaryConverters = {
  "default": function(value) {
    return value.toString();
  }
};
fieldConverters = {
  0: stringConverters,
  1: binaryConverters
};
Query.Field = (function() {
  function Field(msg) {
    this.name = msg.name;
    this.tableId = msg.tableId;
    this.tableFieldIndex = msg.tableFieldIndex;
    this.typeId = msg.typeId;
    this.type = msg.type;
    this.size = msg.size;
    this.modifier = msg.modifier;
    this.formatCode = msg.formatCode;
    this.convert = fieldConverters[this.formatCode][this.type] || fieldConverters[this.formatCode]["default"];
  }
  return Field;
})();
module.exports = Query;